_.namespace('marketstar');

/**
 * Observer proxy for window.localStorage.
 *
 * @class LocalStorageObserverProxy
 * @constructor
 * @uses Constants
 * @uses UtilsMixins
 */
marketstar.LocalStorageObserverProxy = function() {
    if ( arguments.callee._singletonInstance ) {
        return arguments.callee._singletonInstance;
    }
    arguments.callee._singletonInstance = this;
    
    $.extend(marketstar.LocalStorageObserverProxy.prototype, UtilMixins);
    
    observers = {};
    
    
    //////////////////////////////////////////
    //
    // Privileged meths
    
    /**
     * Register an observer 
     *
     * @method registerObserver
     * @param key      {String}
     * @param observer {Object}
     * @param fun      {Function}
     */
    this.registerObserver = function(key, observer, fun) {
        if(!observers[key]) {
            observers[key] = [];
        }
        observers[key].push({
             observer: observer
            ,fun: fun
        });
    };

    /**
     * Fire the observer fun
     * 
     * @method observe
     * @param key       {String}
     * @param eventType {String}
     */
    this.observe = function(key, eventType) {
        if(observers[key]) {
            for(var i =0; i<observers[key].length; i++) {
                observers[key][i].fun.apply(
                     observers[key][i].observer
                    ,[{
                         key:    key
                        ,target: this.readFromLocalStorage(key)
                        ,type:   eventType
                    }]
                );
            }
        }
    };

    /** 
     * Write to local storage.  Handle objects.
     * 
     * @method saveToLocalStorage
     * @param key   {String}
     * @param value {Object}
     */
    this.saveToLocalStorage = function(key, value) {
        try {
            window.localStorage.setItem(key, typeof value === 'object' ? JSON.stringify(value) : value);
        } catch (error){return false;}
    };

    /** 
     * Delete from local storage.
     *
     * @method deleteFromLocalStorage
     * @param key {String}
     */
    this.deleteFromLocalStorage = function (key) {
        window.localStorage.removeItem(key);
    };

    /**
     * Read from local storage.  Handle objects.
     *
     * @method readFromLocalStorage
     * @param key {String}
     */
    this.readFromLocalStorage = function(key) {
        var value = window.localStorage.getItem(key);
        return this.isJsonParsable(value) ? JSON.parse(value) : value;
    };
    
};

// some statics
marketstar.LocalStorageObserverProxy.prototype.UPDATE_EVENT = 'updated';
marketstar.LocalStorageObserverProxy.prototype.DELETE_EVENT = 'deleted';

/**
 * Save to local storage. Optionally, register an observer.
 *
 * @method save 
 * @param key       {String}
 * @param value     {Object}
 * @param observer  {Object}
 * @param fun       {Function}
 * @param overwrite {Boolean}
 */
marketstar.LocalStorageObserverProxy.prototype.save = function(key, value, observer, fun, overwrite) {
    var  exists = this.contains(Object.keys(window.localStorage),key)
        ,overwrite = overwrite || true;
    
    if(exists && !overwrite) {
        return;
    }
    this.saveToLocalStorage(key, value);
    if(observer && fun) {
        this.registerObserver(key, observer, fun);
    }
    if(exists) {
        // only fire the observe if this is an update
        this.observe(key, this.UPDATE_EVENT);
    }
};

/**
 * Read from local storage/
 *
 * @method read
 * @param key {String}
 */
marketstar.LocalStorageObserverProxy.prototype.read = function(key) {
    // TODO: refactor or keep for completeness
    return this.readFromLocalStorage(key);
}

/** 
 * Delete from local storage.
 *
 * @method delete
 * @param key {String}
 */
marketstar.LocalStorageObserverProxy.prototype.delete = function(key) {
    this.deleteFromLocalStorage(key);
    this.observe(key, this.DELETE_EVENT);
};

/**
 * Clear the entire local storage
 *
 * @method clear
 */
marketstar.LocalStorageObserverProxy.prototype.clear = function() {
    // TODO: flag to delete all observers first?
    var  i
        ,keys = Object.keys(window.localStorage)
    for(i=0; i<keys.length; i++){
        this.delete(keys[i]);
    }
};

marketstar.LocalStorageObserverProxy.prototype.getItem = function(key, id){
    try {
        return this.readFromLocalStorage(key).find(function (i) {
            return i.Id == id;
        });
    } catch(e) { return null; }
}

