/**
 * Mixins for making AJAX calls to Jive and Marketstar API
 *
 * @module AjaxMixins
 * @uses Constants
 * @uses UtilMixins
 */
var AjaxMixins = {

    jiveApiLoginPath:                       'people/username/{username}'
    ,jiveApiActivitiesPath:                 'activities'
    ,jiveApiUserStreamsPath:                'people/{userId}/streams'
    ,jiveApiStreamActivitiesPath:           'streams/{streamId}/activities'
    ,jiveApiUserSearchPath:                 'search/people?filter=search({username})&fields=jive.username,thumbnailUrl,name.formatted'

    ,marketstarDefaultLang:                  'en'
    ,marketstarApiUserInfoPath:              'user/info'
    ,marketstarApiDailyQuestionPath:         'dailyquestions'
    ,marketstarApiPointsGiftedPath:          'user/pointsgifted'
    ,marketstarApiTrainingsPath:             'trainings?lang={lang}'
    ,marketstarApiTrainingTypesPath:         'trainingTypes?lang={lang}'
    ,marketstarApiTrainingsByTypePath:       'trainings/bytype/{typeId}?lang={lang}'
    ,marketstarApiAchievement:               'achievements'
    ,marketstarApiAchievementByTrainingName: 'achievements/{trainingName}'
    ,marketstarApiAchievementRatingsPath:    'achievements/{trainingId}/ratings'
    ,marketstarApiUpdateLoginDatePath:       'lastlogin?userId={userId}'

    ,marketstarApiProductCategoriesPath:     'products/categories'
    ,marketstarApiProductsPath:              'products'
    ,marketstarApiProductsByCategoryPath:    'products?categoryId={categoryId}'
    ,marketstarApiUserPermissions:           'user/permissions'

    ,loaderSelector:                         '.user-bar-container .loader'
    ,$loader: null
    ,loaderRequest: []

    /**
     * Get the Jive Api Url
     * @returns {string}
     */
    ,jiveApiUrlPrefix: function() {
        return Config.jiveUrl + '/api/core/v3/';
    }

    /**
     * Get the MarketStar Api Url
     * @returns {string}
     */
    ,marketstarApiUrlPrefix: function() {
        return Config.marketstarUrl + 'api/';
    }

    /**
     * Get the Forgot UserName Url
     * @returns {string}
     */
    ,forgotUserNameURL: function() {
        return Config.jiveUrl + '/forgot-username!input.jspa';
    }

    /**
     * Get the Forgot Password Url
     * @returns {string}
     */
    ,forgotPasswordURL: function() {
        return Config.jiveUrl + '/emailPasswordToken!input.jspa';
    }

    /**
     * Get the Create Account Url
     * @returns {string}
     */
    ,createAccountURL: function() {
        return Config.jiveUrl + '/validate-email.jspa';
    }

    /**
     * Update the src image at selector with the binary contents of the image at url
     *
     * @method updateImageFromAjax
     * @async
     * @param selector {String} The selector for $()
     * @param url      {String} The url for the image
     */
    ,updateImageFromAjax: function(selector, url) {
        this.basicAuthAjaxCall({
             url:       url
            ,type:      'GET'
            ,basicAuth: this.model.get(this.userDataKey).authHash
            ,modelName: url
            ,isBinary:  true
            ,selector:  selector
            ,success:   $.proxy(this.updateImageSrc, this)
            ,fail:      $.proxy(this.failure, this)//this.fail
        });
    }

    /**
     * Handle Failure
     */
    ,failure: function(){
        alert("failure");
    }

    /**
     * Ajax success callback from updateImageFromAjax
     * Updates the src of the image declared in the requestProps (by selector) with the base64'd binary image data.
     *
     * @method updateImageSrc
     * @param requestProps {Object} Hash of the request properties that generated the request
     * @param xhr          {Object} The XmlHttpRequest
     */
    ,updateImageSrc: function(requestProps, xhr) {
        this.$(requestProps.selector).attr( 'src', 'data:image/png;base64,' +  this.byteArrayToBase64(this.model.get(requestProps.modelName)) );
    }

    /**
     * Show the loader in the user-bar
     *
     * @method showLoader
     */
    ,showLoader: function() {
        this.$loader = this.$loader || $(this.loaderSelector);
        this.$loader.show();
    }

    /**
     * Hide the loader in the user-bar
     *
     * @method hideLoader
     */
    ,hideLoader: function() {
        this.$loader = this.$loader || $(this.loaderSelector);
        this.$loader.hide();
    }

    /**
     * Execute an ajax call with basic authentication
     *
     * @method basicAuthAjaxCall
     * @async
     * @param requestProps {Object} Struct of request properties. Must contain: url, type, username, password.
     */
    ,basicAuthAjaxCall: function(requestProps) {
        !requestProps.showLoader ? requestProps.showLoader = true : true; // Default showLoader to true. CMS
        var xhr = new XMLHttpRequest()
            ,async = requestProps.async == null ? true : requestProps.async
            ,that = this;

        if (async) {
            xhr.timeout = Config.globalHttpRequestTimeout;
            xhr.ontimeout = function() {
                that.requestTimeout(that);
            }
        }
        xhr.onerror = function(e) {
            that.requestError(that);
        }
        if(requestProps.username && requestProps.password) {
            var  username  = requestProps.username
                ,password  = requestProps.password
                ,basicAuth = this.generateBaseAuth(username, password);

            delete requestProps.username;
            delete requestProps.password;
            requestProps.authHash = basicAuth;
        } else if(requestProps.basicAuth) {
            var basicAuth = requestProps.basicAuth;
        } else {
            // UNAUTHORIZED
            return;
        }
        xhr.open(requestProps.type, requestProps.url, async);
        xhr.setRequestHeader('Authorization', basicAuth);
        if(requestProps.contentType)
            xhr.setRequestHeader("Content-Type", requestProps.contentType);

        requestProps.xhr = xhr;
        this.ajaxCall(requestProps);
    }

    /**
     * Execute an ajax call against the Marketstar REST API.
     * A valid user object must exist in the sudo model.
     *
     * @method basicAuthAjaxCall
     * @async
     * @param requestProps {Object} Struct of request properties. Must define: url, type.
     */
    ,marketstarApiAjaxCall: function(requestProps) {
        if(requestProps.showLoader == null){requestProps.showLoader = true;} // Default showLoader to true. CMS
        var xhr       = new XMLHttpRequest()
            , user      = this.model.get(this.userDataKey)
            , timestamp = Math.floor( new Date().getTime() / 1000 )
            , async = requestProps.async == null ? true : requestProps.async
            , that = this;

        if(async) {
            xhr.timeout = Config.globalHttpRequestTimeout;
        }
        xhr.ontimeout = function() {
            that.requestTimeout(that);
        }
        xhr.onerror = function(e) {
            that.requestError(that);
        }
        if(user) {
            xhr.open(requestProps.type, requestProps.url, async);
            xhr.setRequestHeader( 'ApiKey',    Config.marketstarApiKey );
            xhr.setRequestHeader( 'UserId',    user.id );
            xhr.setRequestHeader( 'Timestamp', timestamp );
            xhr.setRequestHeader( 'Signature', this.generateMarketstarApiSignature(timestamp, user.id) );
            xhr.setRequestHeader("If-None-Match", "\"doesnt-match-anything\"");  //force refresh
            requestProps.xhr = xhr;
            this.ajaxCall(requestProps);
        }
    }

    /**
     * Build a url to MarketStar API to be used in anchor and/or form actions.
     *
     * @method buildMarketStarConnectUrl
     * @param url {String} url to build
     * @return {string} url with appropriate keys and timestamps
     */
    ,buildMarketStarUrl: function (url, queryParams) {
        var connectUrl = Config.marketstarUrl + url + "?"
            , user = this.model.get(this.userDataKey)
            , timestamp = Math.floor( new Date().getTime() / 1000 )
            , queryString = "";

        if(queryParams) {
            for(var p in queryParams) {
                queryString += (queryString == "" ? "" : "&") + (p + "=" + queryParams[p]);
            }
        }

        queryString += (queryString == "" ? "" : "&") +
            "UserId=" + user.id +
            "&ApiKey=" + Config.marketstarApiKey +
            "&Timestamp=" + timestamp +
            "&Signature=" + this.generateMarketstarApiSignature(timestamp, user.id);

        return connectUrl + queryString;
    }

    /**
     * Execute a generic ajax call.
     *
     * @method basicAuthAjaxCall
     * @async
     * @param requestProps {Object} Struct of request properties. Must define: url, type. Optionally: modelName, success, fail.
     */
    ,ajaxCall: function(requestProps) {
        if(requestProps.showLoader) {
            this.loaderRequest.push(requestProps);
            this.showLoader();
        }
        var  xhr
            ,that = this
            ,async = requestProps.async == null ? true : requestProps.async;

        if(!requestProps.xhr && (requestProps.url && requestProps.modelName)) {
            xhr = new XMLHttpRequest();
            xhr.timeout = Config.globalHttpRequestTimeout;
            xhr.ontimeout = function() {
                that.requestTimeout(that);
            }
            xhr.onerror = function(e) {
                that.requestError(that);
            }
            xhr.open(requestProps.type, requestProps.url, async);
        } else {
            xhr = requestProps.xhr;
        }
        if(requestProps.isBinary) {
            xhr.responseType = 'arraybuffer';
        }
        xhr.onload = function() {
            if(xhr.readyState == 4) {
                if(requestProps.modelName) {
                    that.setModelFromXhr(xhr, requestProps);
                } else {
                    that.ajaxComplete(xhr, requestProps);
                }
            }
        };
        if(requestProps.type == 'POST' && (requestProps.body || requestProps.bodyString))  {
            if(requestProps.body) {
                requestProps.bodyString = JSON.stringify(requestProps.body);
            }
            xhr.setRequestHeader('Content-type','application/json; charset=utf-8');
            // For security reason, browsers will not set this header. CMS xhr.setRequestHeader("Content-length", requestProps.body.length);
            xhr.send(requestProps.bodyString);
        } else if(requestProps.payload){
            xhr.send(requestProps.payload);
        }
        else{
            try {
                xhr.send();
            }
            catch (e) {
                console.log('###### Error xhr.send() ######: ' + e.message);
            }
        }
    }

    /**
     * Execute a generic ajax call.

     *
     * @method basicAuthAjaxCall
     * @async
     * @param xhr          {Object} The XmlHttpRequest
     * @param requestProps {Object} Struct of request properties.
     */
    ,setModelFromXhr: function(xhr, requestProps) {
        var ajax = ajax || false;
        if( xhr.status == 200 || xhr.status == 201 || xhr.status == 0 || (ajax && ajax.status == 0) ) {
            var data;
            if(requestProps.isBinary) {
                var windowUrl = window.URL || window.webkitURL;
                data = new Uint8Array(xhr.response);
            } else if(requestProps.isFile) {
                data = xhr.response;
            } else {
                try {
                    data = JSON.parse(this.cleanJson(xhr.responseText));
                }
                catch (e) {
                    data = xhr.response;
                }
            }
            if(requestProps.modelName) {
                this.model.set(requestProps.modelName, data);
            }
            if(requestProps.success) {
                requestProps.success(requestProps, xhr);
            }
        } else if(requestProps.fail) {
            requestProps.fail(requestProps, xhr);
        } else {
            // default fail?
        }
        if(requestProps.showLoader) {
            this.removeLoaderRequest(requestProps);
        }
    }

    ,ajaxComplete: function(xhr, requestProps) {
        if(xhr.status == 200 || xhr.status == 201 ||xhr.status == 0 /*|| (ajax && ajax.status == 0)*/) {
            if(requestProps.success) {
                requestProps.success(requestProps, xhr);
            }
        } else if(requestProps.fail) {
            requestProps.fail(requestProps, xhr);
        }
        if(requestProps.showLoader) {
            this.removeLoaderRequest(requestProps);
        }
    }

    ,removeLoaderRequest: function(requestProps) {
        if(this.loaderRequest.length == 0) {
            return;
        }
        var lengthBefore = this.loaderRequest.length;
        for(var i = 0; i < this.loaderRequest.length; i++) {
            if(this.loaderRequest[i].url == requestProps.url) {
                this.loaderRequest.splice(i,1);
                break;
            }
        }
        if(lengthBefore > this.loaderRequest.length && this.loaderRequest.length == 0) {
            var that = this.parent || this;
            that.hideLoader();
            that.$el.trigger('doneLoading');
        }
    }

    /**
     * Generalized method to clean up JSON response payload from Jive.
     *
     * @method cleanJson
     * @param json {String}
     * @return {String} shiny clean JSON
     */
    ,cleanJson: function(json) {
        return this.hasValidJsonOpening(json) ? json : this.cleanJiveJson(json);
    }

    /**
     * Remove the non-standard throws statement from Jive REST API JSON results.
     *
     * @method cleanJiveJson
     * @param json {String}
     * @return {String} clean JSON
     */
    ,cleanJiveJson: function(json) {
        return json.substring(json.indexOf('{'));
    }

    /**
     * Determine is the JSON starts with a valid char
     *
     * @method hasValidJsonOpening
     * @param json {String}
     * @return {Boolean}
     */
    ,hasValidJsonOpening: function(json) {
        return json[0] == '[' || json[0] == '{';
    }

    /**
     * Generate the Marketstar REST API Signature header content
     *
     * @method generateMarketstarApiSignature
     * @param timestamp {Number}
     * @return {String} MD5 string of the sig
     */
    ,generateMarketstarApiSignature: function(timestamp, userId) {
        return CryptoJS.SHA1(userId + Config.marketstarApiKey + Config.marketstarSecretKey + timestamp).toString(CryptoJS.enc.Hex);
    }

    /**
     * Generate HTTP basic authentication
     *
     * @method generateBaseAuth
     * @param username {String}
     * @param password {String}
     * @return base64 basic auth string
     */
    ,generateBaseAuth: function(username, password) {
        var  tok = username + ':' + password
            ,hash = btoa(tok);
        return "Basic " + hash;
    }

    /**
     * Builds a url from a struct.
     *
     * @method buildUrl
     * @uses UtilMixins.gsub
     * @param urlParams {Object} Must contain: url or both urlPrefix and urlPath.  Expects '/'s to be included.
     */
    ,buildUrl: function(urlParams) {
        var s = urlParams.url ? '{url}' : '{urlPrefix}{urlPath}';
        return this.gsub(s, urlParams);
    }

    ,requestTimeout: function(scope) {
        scope.hideLoader();
        scope.showError("Internet connection lost. Please try again later.");
    }

    ,requestError: function(scope) {
        scope.hideLoader();
        scope.showError("Internet connection lost. Please try again later.");
    }
};
