/**
 * Mixin funs for page specific behavior.
 *
 * @module PageMixins
 * @uses Constants
 * @uses UtilMixins
 * @uses LocalStorageObserverProxy
 */
PageMixins = {
    /**
     * Load object from local storage at key into sudo model.
     * Supports success and failure callback funs.
     * Expects this.localStorage
     *
     * @method loadIntoModelFromLocalStorage
     * @param key       {String}
     * @param [success] {Function} Success callback
     * @param [fail]    {Function} Failure callback
     */
    loadIntoModelFromLocalStorage: function (key, success, fail) {
        var data = this.localStorage.read(key);
        if (data) {
            this.model.set(key, data);
            if (success) {
                success(key, data);
            }
        } else if (fail) {
            fail(key);
        }
    }

    /**
     * Update the sudo model from local storage.
     * Used as a obvserver callback on objects stored in local storage.
     *
     * @method updateModelFromLocalStorage
     * @param hash {Object}
     */
    ,updateModelFromLocalStorage: function (hash) {
        if (hash.type == this.localStorage.UPDATE_EVENT) {
            this.model.set(hash.key, hash.target);
        }
    }

    /**
     * Redirect to the index page.
     *
     * @method redirectToIndex
     */
    ,redirectToIndex: function () {
        window.location.href = 'index.html';
    }

    /**
     * Register callbacks for teplate fetching on pages that use sudo templates
     *
     * @method registerTemplateCallbacks
     */
    ,registerTemplateCallbacks: function () {
        this.$el.on(this.templateLoadedEventName, $.proxy(this.onTemplateLoaded, this));
        // $.one is a bit of a hack but prevents multiple calls when nested sudo children
        // fetch their own templates
        this.$el.one('templatesReady', $.proxy(this.preTemplatesReady, this));
    }

    /**
     * Genericized event handler for the templates ready event.
     * Ensures the event source is the caller (this).
     *
     * @method preTemplatesReady
     * @param e      {Event}
     * @param source {Object}
     */
    ,preTemplatesReady: function (e, source) {
        if (source && source === this) {
            this.templatesReady();
        }
        this.renderConfigElements();
    }

    /**
     * Renders page elements with associated data from config. CMS
     *
     * @method renderConfigElements
     */
    ,renderConfigElements: function () {
        Config.globalPageTitle ? $('title').html(Config.globalPageTitle) : null;
    }

    /**
     * Load the templates for the page.
     * Templates to be loaded are declared in the sudo model path 'templatePaths'
     * Template contents are then stored in the sudo model path 'templates'
     *
     * @method loadTemplates
     * @async
     */
    ,loadTemplates: function () {
        this.registerTemplateCallbacks();
        var templatePaths = this.model.get('templatePaths');

        if (templatePaths) {
            for (key in templatePaths) {
                this.ajaxCall({
                    url: this.baseTemplatePath + '/' + templatePaths[key], type: 'GET', isFile: true, modelName: templatePaths[key], templateKey: key, success: $.proxy(this.storeTemplateInModel, this), fail: this.fail
                });
            }
        }
    }

    /**
     * Stores a single template in the sudo model using the path 'templates.<template key>'
     * where the template key is the unique name key defined for the template in 'templatePaths'.
     * Deletes the template entry keyed by template filepath stored in the model by the ajax call.
     * Registered as the success callback for loadTemplates.
     * Raises a custom template loaded event.
     *
     * @method storeTemplateInModel
     * @param requestProps {Object}
     * @param xhr          {xhr}
     */
    ,storeTemplateInModel: function (requestProps, xhr) {
        var key = requestProps.templateKey
            , templatePath = requestProps.modelName;

        if (!this.model.get(this.templatesModelKey)) {
            this.model.set(this.templatesModelKey, {});
        }
        this.model.get(this.templatesModelKey)[key] = this.model.get(templatePath);
        //delete this.model.data[templatePath];
        this.$el.trigger(this.templateLoadedEventName, {templateKey: key, source: this});
    }

    /**
     * Event handler for the custom template loaded event.
     * Fires the custom templates ready event when all templates are loaded.
     *
     * @method onTemplateLoaded
     * @event template loaded event (see Constants for event name)
     * @param e           {Object}
     * @param templateKey {String} The unique template key
     * @uses UtilMixins.hashSize
     */
    ,onTemplateLoaded: function (e, params) {
        if (params.source === this) {
            if (this.hashSize(this.model.get(this.templatesModelKey)) == this.hashSize(this.model.get(this.templatePathsModelKey))) {
                this.$el.trigger(this.templatesReadyEventName, this);
            }
        }
    }

    /**
     * Updates the contents of a <ul> from a hash.
     * The hash must define a collection and related properties to render all <li>'s.
     *
     * @method updateUl
     * @params props {Object} Props defining the <li>'s.
     *   Must contain: selector, coll, idField, nameField.
     *   Optional: nameFun, onChangeHandler, initialLoadFun, defaultElement.
     */
    ,updateUl: function (props) {
        var i
            , idValue
            , innerHtml
            , ra = []
            , types = []
            , selector = props.selector        // selector defining the <ul>
            , coll = props.coll            // the collection
            , idField = props.idField         // the id field
            , nameField = props.nameField       // the name field
            , nameFun = props.nameFun         // a fun defining the name, optional but overrides nameField
            , onChangeHandler = props.onChangeHandler // the onChange event handler fun
            , initialLoadFun = props.initialLoadFun  // the fun for initial load, e.g. load first selected/default item in ddl
            , defaultElement = props.defaultElement; // default or first element in ddl


        if (!coll || coll.length == 0) {
            return;
        } // nothing doing

        if (defaultElement) {
            ra.push(defaultElement);
        }

        for (i = 0; i < coll.length; i++) {
            ra.push(coll[i]);
        }

        $(selector).html('');
        for (i = 0; i < ra.length; i++) {
            $(selector).append(
                this.gsub(
                    '<option value="{id}">{innerHtml}</option>', // TODO: move to consts?
                    {
                        id: ra[i][idField], innerHtml: nameFun ? nameFun(ra[i]) : ra[i][nameField]
                    }
                )
            );
        }

        if (initialLoadFun) {
            initialLoadFun(ra[0][idField]);
        }

        $(selector).on('change', $.proxy(onChangeHandler, this));
    }

    /**
     * Parses query parameters into a hash in the sudo model.
     *
     * @method parseQueryParametersIntoModel
     */
    ,parseQueryParametersIntoModel: function () {
        var i
            , kv
            , params = {}
            , queryParams = window.location.search;
        queryParams = (queryParams[0] === '?' ? queryParams.substring(1) : queryParams).split(/&/);
        for (i = 0; i < queryParams.length; i++) {
            kv = queryParams[i].split(/=/);
            params[kv[0]] = kv[1];
        }
        this.model.set(this.queryParameterModelKey, params);
    }

    ,fail: function (e) {
        console.log('FAIL:');
        console.log(e.code);
    }

    ,goBack: function () {
        window.history.back();
    }

    ,anchorClickHandler: function (e) {
        e.preventDefault();
        e.stopPropagation();
        window.location.href = $(e.currentTarget).attr('href');
        return false;
    }

    /**
     * Click event handler attached to an anchor to open a url in the in app browser
     *
     * @method openUrlInAppBrowser
     * @event click
     * @param e {Object} Click event
     * @return {Boolean} Always returns false
     */
    ,openAnchorInAppBrowser: function (e) {
        e.preventDefault();
        e.stopPropagation();
        
        var url = $(e.target).attr('href') || $(e.target).parent().attr('href') || $(e.target).data('href'); // Attempt to fix duplicate clicks on training modules "Take Training" button. CMS
        var contentUrl = $(e.target).data('content-url') || $(e.target).parent().data('content-url');
        
        if (contentUrl != null) {
            // if (contentUrl.substring(contentUrl.length - 4, contentUrl.length) == ".pdf") {
            //     var googleDocsUrl = "http://docs.google.com/viewer?url=" + contentUrl;
            //     var ref = window.open(googleDocsUrl, '_blank', 'location=yes,closebuttoncaption=back');
            // }
            // else {
            var ref = window.open(contentUrl, '_blank', 'location=yes');
            // }
        }
        else if (url != null) {
            var ref = window.open(url, '_blank', 'location=yes');
        }
    }

    /**
     * Click event handler attached to an anchor to open a url in the system browser
     *
     * @method openUrlInSystemBrowser
     * @event click
     * @param e {Object} Click event
     * @return {Boolean} Always returns false
     */
    ,openAnchorInSystemBrowser: function (e) {
        e.preventDefault();
        e.stopPropagation();
        
        var url = $(e.target).attr('href') || $(e.target).parent().attr('href');
        var contentUrl = $(e.target).data('content-url') || $(e.target).parent().data('content-url');
        
        if (contentUrl != null) {
            // if (contentUrl.substring(contentUrl.length - 4, contentUrl.length) == ".pdf") { // && navigator.app) {
            //     var googleDocsUrl = "http://docs.google.com/viewer?url=" + contentUrl;
            //     // navigator.app.loadUrl(googleDocsUrl, {openExternal: true});
            //     var ref = window.open(googleDocsUrl, '_system', 'location=yes');
            // }
            // else {
            var ref = window.open(contentUrl, '_system', 'location=yes');
            // }
        }
        else if (url != null) {
            var ref = window.open(url, '_system', 'location=yes');
        }
    }

    /**
     * Click event handler to open a url in the in app browser
     *
     * @method openUrlInAppBrowser
     * @event click
     * @param url {String} Url to open
     */
    ,openUrlInAppBrowser: function (url) {
        var ref = window.open(url, '_blank', 'location=no');
    }


    /**
     * Render the page navigation.
     *
     * @method renderNavigation
     */
    ,renderNavigation: function () {
        this.renderNavigationTemplate();
        this.renderSocialLinksTemplate();
        this.renderAppVersionTemplate();
        this.navMenuControl();
        this.navMenuLinkHandler();
        this.renderNavNotifications();
        this.renderUserBar();
    }

    /**
     * show error message
     */
    ,showError: function (error, showTime) {
        var that = this
            , $error = this.$('.ui-error');

        $error.html(error);
        $error.slideDown(400, function () {
            setTimeout(that.hideError, (showTime ? showTime : 2500));
        });
    }

    /**
     * hide error message
     */
    ,hideError: function () {
        this.$('.ui-error').hide();
    }

    /**
     * show message
     */
    ,showMessage: function (message) {
        var that = this
            , $error = this.$('.ui-message').length == 0 ? this.parent.$('.ui-message') : this.$('.ui-message');
        $error.html(message);
        $error.slideDown(400, function () {
            setTimeout(that.hideMessage, 2500);
        });
    }

    /**
     * hide message
     */
    ,hideMessage: function () {
        this.$('.ui-message').hide();
    }

    /**
     * Render the navigation badges to show new content within the different menus
     *
     * */
    ,renderNavNotifications: function(){
        var urlParams = {
            urlPrefix:  this.marketstarApiUrlPrefix()
            ,urlPath:    'trainings/GetUpdatedCount'
        };

       this.marketstarApiAjaxCall({
            url:            this.buildUrl(urlParams)
            , type:         'GET'
            , modelName:    'TrainingCount'
            , showLoader:   false
            , success:      function(requestProps, xhr){
               AjaxMixins.hideLoader();
                if(xhr.response != 0) {
                    $('.nav-badge.achievements').html(xhr.response);
                    $('.nav-badge.achievements').show();
                }
                else{
                    $('.nav-badge.achievements').hide();

                }
            }
            , fail:         function(requestProps, xhr){}
        });
    }

    /**
     * Render the top bar showing the menu button, preloader, etc.
     * Requires that the userBar template be included at the sudo path 'templates.userBar'
     */
    ,renderUserBar: function (showButtons) {
        var dataView = new _.DataView(
            this.model.getPath('templates.userBar')
            , {
                renderTarget: this.$('div.user-bar-container'), autoRender: true
            }
        );
        this.addChild(dataView);
        if (showButtons === false) {
            this.$('.user-bar-container .user-bar').hide();
        }
    }

    /**
     * Render the static navigation template.
     * Requires that the navigation template be included at the sudo path 'templates.navigation'
     * Assumes the navigation template is static.
     *
     * @method renderNavigationTemplate
     */
    ,renderNavigationTemplate: function () {
      this.localStorage = new marketstar.LocalStorageObserverProxy();
      var permissions = this.localStorage.read(Constants.permissionsKey);
      var securityGroups = this.localStorage.read(Constants.securityGroupsKey);
      var inVirtualExpertGroup = false;
      try { securityGroups.forEach(function(g) { if (g.name === 'Virtual Expert') inVirtualExpertGroup = true; }); } catch(e) {}
      
      var that = this;
        
      $.each(Config.navigationItems, function (index, item) {
          var data = {
            renderTarget: $('.nav-list'),
            autoRender: true,
          };
          
          var template = _.template(that.model.getPath('templates.navigation'), item);
          
          var itemView = new _.DataView(
              template,
              $.extend(data, item)
          );
          
          that.addChild(itemView);
      });
      
      if (permissions && permissions.GiftPoints) {
          $('.navItem#giftPoints').show();
      }
      else {
          $('.navItem#giftPoints').hide();
      }
    }

    /**
    * Launch Virtual Expert
    */
    ,launchVirtualExpert: function() {
      this.localStorage = new marketstar.LocalStorageObserverProxy();
      var userData = this.localStorage.read(Constants.userDataKey);
      var userInfo = this.localStorage.read(Constants.userInfoKey);
      var url = 'assets/virtualexpert/index.html#no-expert?userId='+userData.id+'&userName='+encodeURIComponent(userData.displayName)+'&department=RSA';
      if (userInfo.Store) {
        url += '&storeId='+userInfo.Store.Id+'&storeName='+userInfo.Store.Name;
      }
      window.location.href = url;
    }

    /**
     * Render the static social links template.
     * Requires that the socialLinks template be included at the sudo path 'templates.socialLinks'
     * Assumes the socialLinks template is static.
     *
     * @method renderNavigationTemplate
     */
    ,renderSocialLinksTemplate: function () {
        this.addChild(
            new _.DataView(
                this.model.getPath('templates.socialLinks')
                , {
                    renderTarget: this.$('div.share-bar'), autoRender: true
                }
            )
        );
        if (Config.twitterURL.length > 0) {
            $('.twitter')
                .attr('href', Config.twitterURL)
                .show()
                .on('click', $.proxy(this.openAnchorInSystemBrowser, this));
        }
        if (Config.facebookURL.length > 0) {
            $('.facebook').attr('href', Config.facebookURL)
                .show()
                .on('click', $.proxy(this.openAnchorInSystemBrowser, this));
        }
        if (Config.googlePlusURL.length > 0) {
            $('.google-plus').attr('href', Config.googlePlusURL)
                .show()
                .on('click', $.proxy(this.openAnchorInSystemBrowser, this));
        }
        if (Config.rssFeedURL.length > 0) {
            $('.rss').attr('href', Config.rssFeedURL)
                .show()
                .on('click', $.proxy(this.openAnchorInSystemBrowser, this));
        }
    }

    ,renderAppVersionTemplate: function(){
        this.addChild(new _.DataView(this.model.getPath('templates.appVersion'), {renderTarget: this.$('div.app-version'), autoRender:true}));
        $('.appVersion').append('Version ' + Config.appVersion);
    }

    /**
     * Nav menu click controls as well as link tap action.
     */
    ,navMenuControl: function () {
        $('.top-bar').on('click', '.icons-menu-icon', this.navMenuHandler);
        $('.top-bar').on('click', '.icons-create-icon', this.navCreateHandler);
    }
    ,navCreateHandler: function () {
        window.location.href = "createStatusUpdate.html";
        return false;
    }
    ,navMenuHandler: function () {
        var $nav = $('.user-nav');
        if ($nav.hasClass('menu-expanded')) {
            $('.app, .top-bar, .searchbar').animate({
                'left': '0'
            }, 200, function () {
                $nav.removeClass('menu-expanded');

                if (typeof iScroll !== "undefined") {
                    $('body').css('overflow', 'auto').unbind('touchmove')
                }

                $nav.hide();
            });
        } else {
            $nav.show();
            $('.app, .top-bar, .searchbar').animate({
                'left': '75%'
            }, 200, function () {
                $nav.addClass('menu-expanded');

                if (typeof iScroll !== "undefined") {
                    //$nav.css('overflow-y', 'hidden');
                    $('body').css('overflow', 'hidden').bind('touchmove', function (e) {
                        e.preventDefault()
                    });
                }
                else {
                    $nav.css('overflow-y', ' auto');
                }
            });
        }
    }

    ,missionFilterHandler: function () {
        var $filter = $('.filtering-position');

        if($filter.position().left != 0 || $filter.position().top != 0) {
            /*$filter.animate({
                'left': '0'
            }, 500); */
			$('.filter-overlay, .filtering-position').removeClass('hide displayNone').addClass('show');
			/*$('.filter-overlay, .filtering-position').removeClass('hide').addClass('show').delay(1000).queue(function(next){
				$(this).removeClass('displayNone');
				next();
			});*/
        }
        else{
            /*$filter.animate({
                'left': '100%'
            }, 500);*/
			//$('.filter-overlay, .filtering-position').removeClass('show').addClass('hide');
			$('.filter-overlay, .filtering-position').removeClass('show').addClass('hide').delay(500).queue(function(next){
				$(this).addClass('displayNone');
				next();
			});
        }
    }

    ,navMenuLinkHandler: function () {
        $('.nav-list').on('click, tap', 'a', function () {
            var currentUrl = $(this).attr('rel');
            window.location.href = currentUrl;
            return false;
        });
    }

    ,completeLogout: function(option) {
        if(option == 1)
        {
            //window.location.href = 'homepage.html';

        }
        else if(option == 2){
            this.localStorage = new marketstar.LocalStorageObserverProxy();
            this.localStorage.clear();
            window.location.href= 'index.html?loggedout=true';
        }
        else if(option == 3){
            this.localStorage = new marketstar.LocalStorageObserverProxy();
            this.localStorage.clear();
            window.location.href= 'index.html?invalidUser=true';
        }
    }

    ,confirmLogout: function(){
        if(window.device) {
            navigator.notification.confirm(
                    'Are you sure you want to log out?',  // message
                    PageMixins.completeLogout,         // callback
                    'Logout',            // title
                    'Cancel,Yes'                  // buttonName
            );
        }
        else
        {
            var c = window.confirm("Logout?");
            if(c)
            {
                PageMixins.completeLogout(2);
            }
            else
            {
                PageMixins.completeLogout(1);
            }
        }
    }

    ,hideOverlay: function() {
        $('div.overlay').hide();
    }

    ,showOverlay: function() {
        $('div.overlay').show();
        AjaxMixins.hideLoader();
    }

};

jQuery.fn.extend({
    insertAtCaret: function (myValue) {
        return this.each(function (i) {
            if (document.selection) {
                //For browsers like Internet Explorer
                this.focus();
                var sel = document.selection.createRange();
                sel.text = myValue;
                this.focus();
            }
            else if (this.selectionStart || this.selectionStart == '0') {
                //For browsers like Firefox and Webkit based
                var startPos = this.selectionStart;
                var endPos = this.selectionEnd;
                var scrollTop = this.scrollTop;
                this.value = this.value.substring(0, startPos) + myValue + this.value.substring(endPos, this.value.length);
                this.focus();
                this.selectionStart = startPos + myValue.length;
                this.selectionEnd = startPos + myValue.length;
                this.scrollTop = scrollTop;
            } else {
                this.value += myValue;
                this.focus();
            }
        });
    }
});

String.prototype.format = function () {
    var args = arguments;
    return this.replace(/\{(\d+)\}/g, function (m, n) {
        return args[n];
    });
};
