/**
 * Mixin providing utility funs.
 *
 * @module UtilMixins
 */
UtilMixins = {
    
    /**
     * Render the time since phrase, a'la Jive for two dates.
     *
     * @method getTimeSince
     * @param d1 {Object}
     * @param d2 {Object}
     * @return {String} 
     */  
    getTimeSince: function(d1, d2) {
        var  ONE_SECOND = 60 * 1000
            ,ONE_MINUTE = 60 * ONE_SECOND
            ,ONE_HOUR =   60 * ONE_MINUTE
            ,ONE_DAY =    24 * ONE_HOUR
            ,ONE_WEEK =    7 * ONE_DAY
            ,ONE_MONTH =   4 * ONE_WEEK
            ,ONE_YEAR =  365 * ONE_DAY
            ,ms = Math.abs(d1.getTime() - d2.getTime());
        
        if(ms < ONE_MINUTE) {
            return this.getTimespanPhrase( Math.floor(ms / ONE_SECOND), 'second' );
        } else if (ms < ONE_HOUR) {
            return this.getTimespanPhrase( Math.floor(ms / ONE_MINUTE), 'minute');
        } else if (ms < ONE_DAY) {
            return this.getTimespanPhrase( Math.floor(ms / ONE_HOUR), 'hour' );
        } else if (ms < ONE_WEEK) {
            return this.getTimespanPhrase( Math.floor(ms / ONE_DAY), 'day' );
        } else if (ms < ONE_MONTH) {
            return this.getTimespanPhrase( Math.floor(ms / ONE_WEEK), 'week' );
        } else if (ms < ONE_YEAR) { 
            return this.getTimespanPhrase( Math.floor(ms / ONE_MONTH), 'month' );
        } else {
            return this.getTimespanPhrase( Math.floor(ms / ONE_YEAR), 'year' );
        }        
    }
    
    /**
     * Get the timespan phrase
     *
     * @method getTimespanPhrase
     * @param count  {Number}
     * @param period {String} 
     * @return {String}
     */
    ,getTimespanPhrase: function(count, period) {
        return count + ' ' + this.simplePluralize(count, period) + ' ago';
    }
    
    /** 
     * Overly simple pluralize method
     *
     * @method simplePluralize
     * @count {Number}
     * @word  {String}
     * @return {String} Pluralized word, singular if count is 1 or -1.
     */
    ,simplePluralize: function(count, word) {
        return count != 1 || count != -1 ? word + 's' : word;
    }  
    
    /**
     * Convert a byte array to base64 string
     *
     * @method byteArrayToBase64
     * @param binaryData {Object} Uint8Array of data.
     * @param {String} base64 string representation of binaryData
     * @return {String}
     */
    ,byteArrayToBase64: function(binaryData) {
        var s = '';
        for(i=0; i<binaryData.length; i++) {
            s += String.fromCharCode(binaryData[i]);
        }    
        return btoa(s);
    }
    
    /**
     * Rubyish gsub fun.
     * Uses a hash to drive substring replacements. Replacement keys must be surrounded by {}.
     * Replacements are executed in kv order as they appear in the hash.
     *
     * @method gsub
     * @param string              {String} The string to replace on.  
     * @param hash                {Object} The hash containing the kv pairs for replacement.  
     * @param [regexModifiers=ci] {String} 
     * @return {String}
     */
    ,gsub: function(string, hash, regexModifiers) {
        var key;
        regexModifiers = regexModifiers || 'gi';
        for(key in hash) {
            string = string.replace( new RegExp('\{'+key+'\}', regexModifiers), typeof hash[key] == 'object' ? JSON.stringify(hash[key]) : hash[key] );
        }
        return string;
    }
        
    /**
     * Compute the size of a hash, i.e. count of kv pairs
     * 
     * @method hashSize
     * @param hash {Object}
     * @return {Number}
     */
    ,hashSize: function(hash) {
        if (!hash) { return 0; }
        return Object.keys(hash).length;
    }
    
    /**
     * Util to check for contains in array
     *
     * @method contains
     * @param ra   {Object}
     * @param elem {Object}
     * @return {Boolean}
     */
    ,contains: function(ra, elem) {
        if(!ra || !Array.isArray(ra)) {return false;}
        var  i = ra.length;
        while (i--) {
            if (ra[i] === elem) {
                return true;
            }
        }
        return false;
    }
    
    /** 
     * Find the first eleme in coll where elem[field] == value
     *
     * @method find
     * @param coll   {Object} 
     * @param field  {String} 
     * @param value  {Array}  
     * @return {Object}
     */
    ,find: function(ra, field, value) {
        var results = this.where(ra, field, value);
        if(!results || results.length == 0) {
            return null;
        }
        return results[0]
    }
    
    /** 
     * Find all eleme's in coll where elem[field] == value
     *
     * @method where
     * @param coll   {Object} 
     * @param field  {String} 
     * @param value  {Array}  
     * @return {Array}
     */
    ,where: function(coll, field, value) {
        return this.map(coll, function(elem){ 
            return elem[field] == value; 
        });
    }
    
    /**
     * Find all elem's in coll where elem[field] in source
     *
     * @method whereIn
     * @param coll   {Object}
     * @param field  {String} 
     * @param source {Array}  Array of values elem[field] compares against.
     * @return {Array}
     */
    ,whereIn: function(coll, field, source) { 
        return this.map(coll, function(elem){ 
            return UtilMixins.contains(source, elem[field]); 
        });
    }
    
    /**
     * Underscorey pluck fun.
     *
     * @method pluck
     * @param coll  {Object} Elem's must be Objects.
     * @param field {String} Field name in elem to pluck.
     * @return {Array}
     */
    ,pluck: function(coll, field) {
        return this.map(coll, null, function(elem) {
            return elem[field];    
        })  
    }
    
    // TODO: at this point prolly makes sense to replace with _.js
    /**
     * Poor man's map.
     * Supports filtering and mapping.
     *
     * @method map
     * @param coll      {Object} The collection.
     * @param filterFun {Function} Fun to filter elem's in coll with.
     * @param mapFun    {Function} Fun called on each elem.
     * @return {Array}
     */
    ,map: function(coll, filterFun, mapFun) {
        var  i
            ,elem
            ,key
            ,results = [];
        if(Array.isArray(coll)) {
            for(i=0; i<coll.length; i++) {
                elem = coll[i];
                if( filterFun && filterFun(elem) ) {
                    results.push(elem);
                } else if(mapFun && mapFun(elem)) {
                    results.push(mapFun(elem));
                }
            } 
        } else {
            for(key in coll) {
                elem = coll[key]
                if( filterFun && filterFun(elem) ) {
                    results.push(elem)
                } else if(mapFun && mapFun(elem)) {
                    results.push(mapFun(elem));
                }
            }
        }      
        return results;
    }
    
    /**
     * Simplified check if string is parsable as JSON.
     * Does not test validity.
     *
     * @method isJsonParsable
     * @param s {String}
     * @return {Boolean}
     */
    ,isJsonParsable: function(s) {
        return s ? ( s.substring(0,2) == '"{' ||
                     s.substring(0,2) == '"[' ||
                     s[0] == '{' ||
                     s[0] == '[' ||
                     Number(s) ) : false;
    }
    
    /**
     * Normalize a float into a score of floor, floor.5 or ceil.
     *
     * @method normalizeScore
     * @param scoreF {Number}
     */
    ,normalizeScore: function(scoreF) {
        if(typeof scoreF == 'number') {
            var  fractional
                ,integral;
            scoreF     = scoreF.toFixed(2);
            fractional = scoreF%1; // wut?
            integral   = scoreF - fractional;
            
            switch(true) {
                case fractional < 0.25:
                    fractional = 0;
                    break;
                case fractional < 0.75:
                    fractional = 0.5;
                    break;
                default:
                    fractional = 0;
                    integral += 1;
                    break;
            }
            return fractional + integral;
        }        
    }
    
    ,convertScoreToClassName: function(scoreF) {
        if(typeof scoreF == 'number') {
            var  fractional     = scoreF%1
                ,integralWord   = this.digitToWord(scoreF - fractional);
            return integralWord + (fractional >= 0.5 ? '-half' : '');
        }
    }
    
    ,digitWords: ['zero', 'one', 'two', 'three', 'four', 'five', 'six', 'seven', 'eight', 'nine']
    
    /** 
     * Converts digit to corresponding English word.
     *
     * @method digitToWord
     * @param digit
     * @return {String}
     */
    ,digitToWord: function(digit) {
        if(typeof digit == 'number') {
           return this.digitWords[digit];
        }
    }

    /** 
     * Converts English word to corresponding digit.
     *
     * @method wordToDigit
     * @param word
     * @return {Number}
     */
    ,wordToDigit: function(word) {
        word = word.toLowerCase();
        if(typeof word == 'string') {
            var i;
            for(i=0; i<this.digitWords.length; i++ ) {
                if(this.digitWords[i] == word) {
                    return i;
                }
            }
        }
    }

    ,createUUID: function() {
        // http://www.ietf.org/rfc/rfc4122.txt
        var s = [];
        var hexDigits = "0123456789abcdef";
        for (var i = 0; i < 36; i++) {
            s[i] = hexDigits.substr(Math.floor(Math.random() * 0x10), 1);
        }
        s[14] = "4";  // bits 12-15 of the time_hi_and_version field to 0010
        s[19] = hexDigits.substr((s[19] & 0x3) | 0x8, 1);  // bits 6-7 of the clock_seq_hi_and_reserved to 01
        s[8] = s[13] = s[18] = s[23] = "-";

        var uuid = s.join("");
        return uuid;
    }

    //This will return a given query string parameter from the current url
    ,getQueryParametersByName: function (name) {
        var match = RegExp('[?&]' + name + '=([^&]*)').exec(window.location.search);
        return match && decodeURIComponent(match[1].replace(/\+/g, ' '));
    }
};